#include "GeneralDefinitions.h"

#include "Domain.h"
#include "Equation.h"
#include "Initializer.h"

#include "Path.h"
#include "SemilinePath.h"
#include "SpiralPath.h"
#include "SimplyConnectedDomain.h"

#include "PathSolution.h"
#include "DomainSolution.h"


#include "Stepper.h"
#include "fixedRK4Stepper.h"
#include "Mapper.h"

////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////

///Displays simple progress bar
///\arg [in] Now - how much is left
///\arg [in] Total - general number of events
///\arg [in] width  - how many dots will be used in display
int progressBar( double Now, double Total, int width = 40 );

////////////////////////////////////////////////////////////////////////


///Path type
/// 1 - semiline
/// 2 - spiral
const int  pathType = 1;

/// Domain of integration
const cmplx xLeftUp( -1.5, 1.5 );
const cmplx xRightDown( 1.5, -1.5 );





int main()
{
	cout << "Hello" << endl;
	
	
	//create new mapper
	Mapper  mapper( new Equation(), new Initializer(), new Domain ( xLeftUp, xRightDown ), new fixedRK4Stepper( new Equation(), 1000.0, 0.000001, 0.0001 ) );

	//keeps solutions along paths
	DomainSolution  domainSolution;
	
	//helper variables
	PathSolution * pathSolution;
	
	//Path of integration
	Path * path;

	cout << "Creating paths ..." << endl;
	
	const int NDir = 2000;
	const double r0 = 0.0;
	const double rint = 0.00001;
	const double rstep = 0.01;
	
	double phase = 0.0;
	
	cmplx x( 0.0 );
	
	
	SimplyConnectedDomain< Path > paths; 
	
	for( unsigned long int dir = 0; dir < NDir + 1; dir++ )
	{
		
		phase = double(dir)/double(NDir) * 2.0 * M_PI;
		x = rint * exp( IU * phase );
		
		
		if( pathType == 1 )
		{
			path = new SemilinePath( x, phase, rstep );
		}
		else if( pathType == 2 )
		{
			path = new SpiralPath( 0.0, rstep, phase , 0.5, rint, 1.0 );
		} 
		
		paths.fill( path );
	}	
		
	cout << "...DONE" << endl;	
		
	
	cout << "Start scanning ... " << endl;
	
	
	int  i = 0;
	
	//loop over paths
	while( paths.hasNext() )
	{		
		//get next path
		path = paths.getNextPath();
		
		//integrate 
		pathSolution = mapper.mapPath( path );
		
		//save results
		domainSolution.fill( pathSolution );

		//show simple progress bar
		progressBar( i, paths.getNPaths()-1 );


		i++;
	
	}
	
	cout << endl << "... DONE" << endl;
	
	
	
	cout << endl <<"Saving results..." << endl;
	
		domainSolution.saveToFile( string( "abs.out" ) );
	
	cout << "...DONE" << endl;
	
	
	
	cout << endl;
	
	return 0;
	
};



////////////////////////////////////////////////////////////////////////


int progressBar( double Now, double Total, int width )
{
  
    double fraction = Now / Total;
    // number of = to display
    int N = round(fraction * width);

    
    int i=0;
    printf("%3.0f%% [",fraction*100);
    // done part
    for ( ; i < N;i++) 
    {
        printf("=");
    }
    // remaining part (spaces)
    for ( ; i < width;i++) 
    {
        printf(" ");
    }
    //print end and info
    printf("] ");
    printf(" %.0f of %.0f", Now, Total );
    //go back at the beginning
    printf("\r");
    fflush(stdout);
    
};
